#' @include internal.R Collection-proto.R
NULL

#' Solution portfolio methods
#'
#' Conservation planners often desire a portfolio of solutions
#' to present to decision makers. This is because conservation planners
#' often do not have access to "perfect" information, such as cost data that
#' accurately reflects stakeholder preferences, and so having multiple
#' near-optimal solutions can be a useful.
#'
#' @details All methods for generating portfolios will return solutions that
#'   are within the specified optimality gap. Below are the portfolio methods
#'   that can be added to a \code{ConservationProblem} object.
#'
#'   \describe{
#'   \item{\code{add_default_portfolio}}{Generate a single solution.}
#'
#'   \item{\code{\link{add_cuts_portfolio}}}{
#'     Generate a portfolio of solutions using Bender's cuts.}
#'
#'   \item{\code{\link{add_shuffle_portfolio}}}{
#'     Generate a portfolio of solutions by randomly reordering the data
#'     prior to attempting to solve the problem.}
#'
#'   }
#'
#' @seealso \code{\link{constraints}},  \code{\link{decisions}},
#'  \code{\link{objectives}} \code{\link{penalties}}, \code{\link{problem}},
#'  \code{\link{solvers}}, \code{\link{targets}}.
#'
#' @examples
#' # load data
#' data(sim_pu_raster, sim_features)
#'
#' # create problem
#' p <- problem(sim_pu_raster, sim_features) %>%
#'      add_min_set_objective() %>%
#'      add_relative_targets(0.1) %>%
#'      add_binary_decisions() %>%
#'      add_default_solver(gap = 0.2, verbose = FALSE)
#'
#' # create problem with cuts portfolio
#' p1 <- p %>% add_cuts_portfolio(4)
#'
#' # create problem with shuffle portfolio
#' p2 <- p %>% add_shuffle_portfolio(4)
#' \donttest{
#' # solve problems and create portfolios of solutions within 20 % of optimality
#' # using different emthods
#' s <- list(solve(p1), solve(p2))
#'
#' # plot solutions from cuts portfolio
#' plot(s[[1]], axes = FALSE, box = FALSE)
#'
#' # plot solutions from cuts portfolio
#' plot(s[[2]], axes = FALSE, box = FALSE)
#' }
#'
#' @name portfolios
NULL
